package ru.tularegion.corp

import androidx.compose.foundation.Image
import androidx.compose.foundation.layout.Arrangement
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.Spacer
import androidx.compose.foundation.layout.fillMaxHeight
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.height
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.material3.Icon
import androidx.compose.material3.MaterialTheme
import androidx.compose.material3.NavigationRail
import androidx.compose.material3.NavigationRailItem
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Surface
import androidx.compose.material3.Text
import androidx.compose.runtime.*
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.PasswordVisualTransformation
import androidx.compose.ui.unit.dp
import corp_tularegion_extension.composeapp.generated.resources.Res
import corp_tularegion_extension.composeapp.generated.resources.logo_48
import corp_tularegion_extension.composeapp.generated.resources.round_info_24
import corp_tularegion_extension.composeapp.generated.resources.round_open_in_new_24
import corp_tularegion_extension.composeapp.generated.resources.round_settings_24
import org.jetbrains.compose.resources.DrawableResource
import org.jetbrains.compose.resources.painterResource
import ru.tularegion.corp.backend.JsCallbacks
import ru.tularegion.corp.backend.setAutoAuthLogin
import ru.tularegion.corp.backend.setAutoAuthMode
import ru.tularegion.corp.backend.setAutoAuthPassword
import ru.tularegion.corp.backend.setCopyMode
import ru.tularegion.corp.backend.setRedirectMode
import ru.tularegion.corp.ui.components.CopyMode
import ru.tularegion.corp.ui.components.DropdownSelect
import ru.tularegion.corp.ui.components.RedirectMode
import ru.tularegion.corp.ui.components.SwitchSpoiler
import ru.tularegion.corp.ui.theme.darkScheme

enum class Screen(
    val title: String,
    val icon: DrawableResource,
) {
    Settings(
        title = "Настройки",
        icon = Res.drawable.round_settings_24,
    ),
    About(
        title = "О расширении",
        icon = Res.drawable.round_info_24,
    ),
}

@Composable
fun App() {
    MaterialTheme(
        colorScheme = darkScheme,
    ) {
        Surface {
            Row {
                val navigationState = remember {
                    mutableStateOf(Screen.Settings)
                }
                NavigationRail(
                    containerColor = MaterialTheme.colorScheme.surfaceContainerLow,
                ) {
                    Column(
                        modifier = Modifier
                            .padding(
                                vertical = 8.dp,
                            )
                            .fillMaxHeight(),
                        verticalArrangement = Arrangement.SpaceBetween,
                        horizontalAlignment = Alignment.CenterHorizontally,
                    ) {
                        Column(
                            horizontalAlignment = Alignment.CenterHorizontally,
                        ) {
                            Spacer(
                                modifier = Modifier
                                    .height(8.dp),
                            )
                            Image(
                                painter = painterResource(Res.drawable.logo_48),
                                contentDescription = null,
                            )
                            Spacer(
                                modifier = Modifier
                                    .height(24.dp),
                            )
                            Screen.entries.forEach {
                                NavigationRailItem(
                                    selected = it == navigationState.value,
                                    onClick = {},
                                    icon = {
                                        Icon(
                                            painter = painterResource(it.icon),
                                            contentDescription = null,
                                        )
                                    },
                                )
                            }
                        }
                        NavigationRailItem(
                            selected = false,
                            onClick = {},
                            icon = {
                                Icon(
                                    painter = painterResource(Res.drawable.round_open_in_new_24),
                                    contentDescription = null,
                                )
                            },
                        )
                    }
                }

                Column(
                    modifier = Modifier
                        .verticalScroll(
                            state = rememberScrollState(),
                        )
                        .padding(
                            bottom = 16.dp,
                        )
                        .fillMaxSize(),
                ) {
                    Column(
                        modifier = Modifier
                            .padding(16.dp),
                        verticalArrangement = Arrangement.spacedBy(16.dp),
                    ) {
                        val redirectModeState = remember {
                            mutableStateOf(RedirectMode.NO_REDIRECT)
                        }
                        JsCallbacks.redirectMode = { value ->
                            redirectModeState.value = RedirectMode.entries.find {
                                it.id == value
                            } ?: RedirectMode.NO_REDIRECT
                        }
                        DropdownSelect(
                            title = "Перенаправление между RU и LOCAL",
                            caption = "Перенаправлять все страницы на RU, либо на LOCAL",
                            items = RedirectMode.entries,
                            valueState = redirectModeState,
                            onValueChange = {
                                setRedirectMode(it)
                            },
                        )

                        val copyModeState = remember {
                            mutableStateOf(CopyMode.NO_REPLACE)
                        }
                        JsCallbacks.copyMode = { value ->
                            copyModeState.value = CopyMode.entries.find {
                                it.id == value
                            } ?: CopyMode.NO_REPLACE
                        }
                        DropdownSelect(
                            title = "Замена ссылок при копировании",
                            caption = "Заменять все ссылки при копировании на RU, либо на LOCAL",
                            items = CopyMode.entries,
                            valueState = copyModeState,
                            onValueChange = {
                                setCopyMode(it)
                            },
                        )
                    }

                    val autoAuthModeState = remember {
                        mutableStateOf(false)
                    }
                    JsCallbacks.autoAuthMode = {
                        autoAuthModeState.value = it
                    }
                    SwitchSpoiler(
                        title = "Автоматический вход",
                        caption = "Выполнять вход на портал автоматически",
                        enabledState = autoAuthModeState,
                        onValueChange = {
                            setAutoAuthMode(it)
                        },
                    ) {
                        Column(
                            modifier = Modifier
                                .padding(16.dp),
                            verticalArrangement = Arrangement.spacedBy(16.dp),
                        ) {
                            val loginState = remember {
                                mutableStateOf("")
                            }
                            JsCallbacks.autoAuthLogin = {
                                loginState.value = it
                            }
                            OutlinedTextField(
                                value = loginState.value,
                                onValueChange = {
                                    setAutoAuthLogin(it)
                                    loginState.value = it
                                },
                                modifier = Modifier
                                    .fillMaxWidth(),
                                label = {
                                    Text(
                                        text = "Логин",
                                    )
                                },
                                singleLine = true,
                            )

                            val passwordState = remember {
                                mutableStateOf("")
                            }
                            JsCallbacks.autoAuthPassword = {
                                passwordState.value = it
                            }
                            OutlinedTextField(
                                value = passwordState.value,
                                onValueChange = {
                                    setAutoAuthPassword(it)
                                    passwordState.value = it
                                },
                                modifier = Modifier
                                    .fillMaxWidth(),
                                label = {
                                    Text(
                                        text = "Пароль",
                                    )
                                },
                                visualTransformation = PasswordVisualTransformation(),
                                singleLine = true,
                            )
                        }
                    }
                }
            }
        }
    }
}